/***
    NAME
    cr_season.c

    NOTES
    The problem specific definition file for the size-structured
    consumer-resource model with pulsed reproduction possibly every
    spring, if accumulated fat is sufficient and individuals are
    are mature.

    This program incorporates a summer and winter period, which together not
    necessarily have to add up to 365 days. Summer duration is set by a
    parameter, while winter duration is the difference between the year length
    and the summer duration. Notice that the start of the summer season is
    chosen as day 0 of the year.

    At the end of the summer season the investment into eggs is irreversibly
    allocated. These eggs (or what is left of them after possible winter
    starvation) are spawned in a pulsed event at day SPAWN_DAY of the next
    year. Hatching of eggs immediately follows spawning if PULSED_REPRO is
    unequal to 0. Otherwise, hatching of eggs takes place spread out over a
    period of days following a truncated normal distribution that is fully
    determined by the start of the hatching period (HATCH_START) and the peak
    of the hatching at day HATCH_MAX in the season.

    In this program both a (modified) sinusoidal temperature regime and a step
    function as temperature regime is modelled. The consumer and resource
    response to the temperature regime is parameterized by the relative
    reduction in consumer physiological rates and resource productivity at the
    mid-winter temperature (MIN_TEMP), compared to the default value at the
    moid-summer value (MAX_TEMP). In beween, the Arrhenius relation is used to
    scale these rates to the ambient temperature.

    Terminology:

    X:	Irreversible mass or bone
    Y:	Reversible mass or fat
    M:	Mass or standard mass, (1+QJ)*X
    W:	Weight or total weight: X+Y

    HISTORY
    AMdR - Sep 13, 2008: Last Revision.
***/

#include  "escbox.h"

#define STEP_TEMP	1			// 0/1: Sinusoidal/step temp. function
#define PULSED_REPRO	0			// Pulsed (1) or distributed (0) repro.


/*
 *===========================================================================
 *
 *		LABELLING ENVIRONMENT AND I-STATE VARIABLES
 *
 *===========================================================================
 */
#define	time		(env[0])
#define	food		(env[1])
#define TotalEggs	(env[2])

#define	age		(i_state(0))
#define	bone		(i_state(1))
#define	fat		(i_state(2))

#define	IDmass		(i_const( 0))
#define	IDweight	(i_const( 1))
#define	IDarate		(i_const( 2))
#define	IDhtime		(i_const( 3))
#define	IDmaint		(i_const( 4))
#define	IDingest	(i_const( 5))
#define	IDnet_energy	(i_const( 6))
#define	IDkappa		(i_const( 7))
#define	IDmort		(i_const( 8))
#define	IDfatratio	(i_const( 9))
#define	IDfecundity	(i_const(10))
#define	IDnbegin	(i_const(11))

#define	CONS		0

#define	SMALL		0.001
#define	TINY		1.0E-6
#define MINFOOD		1.0E-12



/*
 *===========================================================================
 *
 *		DEFINING AND LABELLING CONSTANTS AND PARAMETERS
 *
 *===========================================================================
 */
#define MU_BACK		parameter[ 0]	// Background mortality rate
#define MU_J		parameter[ 1]	// Size-dep. mortality constant
#define X_MU		parameter[ 2]	// Size-dep. mortality exponent

#define AR_CON		parameter[ 3]	// Attack rate constant
#define AR_EXP		parameter[ 4]	// Attack rate exponent
#define AR_WOPT		parameter[ 5]	// Optimal consumption weight

#define DIGT_CON	parameter[ 6]	// Digestion time constant
#define DIGT_EXP	parameter[ 7]	// Digestion time exponent

#define MR_CON		parameter[ 8]	// Maintenance allometric constant
#define MR_EXP		parameter[ 9]	// Maintenance allometric exponent

#define CONS_IW		parameter[10]	// Weight of neonate individual
#define XMAT		parameter[11]	// Irreversible mass at maturation

#define QJ		parameter[12]	// Juvenile and adult reversible to
#define QA		parameter[13]	// irreversible mass ratio
#define CONVEFF		parameter[14]	// Food conversion efficiency

#define QS		parameter[15]	// Starvation mortality parameters
#define MU_S		parameter[16]	

#define R		parameter[17]	// Resource growth rate
#define K		parameter[18]	// Pelagic carrying capacity (g/m3)

#define TOTVOL		parameter[19]	// Volume of entire lake
#define SEASON		parameter[20]	// Season duration
#define YEAR		parameter[21]	// Year length

#define SPAWN_DAY	parameter[22]	// Day at which adults spawn eggs

#define HATCH_START	parameter[23]	// Day at which eggs start to hatch
#define HATCH_MAX	parameter[24]	// Day at which most eggs hatch
#define HATCH_SURV	parameter[25]	// Probability that spawned egg hatches

#define MIN_TEMP	parameter[26]	// Mid-winter temperature
#define MAX_TEMP	parameter[27]	// Mid-summer temperature
#define THETA		parameter[28]	// Steepness seasonality shift

#define CONS_MIN_RATE	parameter[29]	// Winter consumer rate reduction
#define RES_MIN_RATE	parameter[30]	// Winter resource rate reduction

#define CONS_MAX_YEARS	25.0



/*
 *===========================================================================
 *
 *		DEFINING GLOBAL VARIABLES AND USER-IMPLEMENTED FUNCTIONS
 *
 *===========================================================================
 */

static void		UpdateIDcards(double *env, population *pop, int setfec);
static double		Temp_factor(double t, double min_fac);

static double		YOYsurvival, SizeOYO, MeanFecundity;
static double		TimeInYear;
static int		ReproCohorts;
static int		NewCohort = -1;



/*
 *===========================================================================
 *
 * USER INITIALIZATION ROUTINE ALLOWS OPERATIONS ON INITIAL POPULATIONS
 *
 *===========================================================================
 */

void	UserInit(int argc, char	**argv, double *env, population *pop)

{
  switch (argc)
    {
    case 5:
      X_MU = atof(argv[4]);
    case 4:
      MU_J = atof(argv[3]);
    case 3:
      MU_BACK = atof(argv[2]);
    default:
      break;
    }

  UpdateIDcards(env, pop, 0);

  ReportNote("\n    %s%s",
#if (BIFURCATION == 1)
	     "Bifurcation of the size-structured consumer-resource model",
#else
	     "Dynamics of the size-structured consumer-resource model",
#endif // BIFURCATION
	     "\n");

  return;
}




/*
 *===========================================================================
 *
 *	SPECIFICATION OF THE NUMBER AND VALUES OF BOUNDARY POINTS
 *
 *===========================================================================
 */

void	SetBpointNo(double *env, population *pop, int *bpoint_no)

{
  bpoint_no[CONS] = 0;

#if (PULSED_REPRO == 0)
  if (TotalEggs > TINY)
    {
      int		Hatching, i;

      TimeInYear = fmod(time, YEAR);
      Hatching   = ((TimeInYear >= (floor(HATCH_START/cohort_limit)*cohort_limit)) &&
		    (TimeInYear <  (ceil((2*HATCH_MAX-HATCH_START)/cohort_limit)*cohort_limit - TINY)));

      // For numerical stability of the integration we use a formulation for
      // the dynamics of the boundary cohort that is different than the
      // standard one
      if (Hatching)
	{
	  NewCohort = AddCohorts(pop, CONS, 1);

	  pop[CONS][NewCohort][number] = 0.0;
	  pop[CONS][NewCohort][age]    = 0.0;
	  pop[CONS][NewCohort][bone]   = 1/(1+QJ)*CONS_IW;
	  pop[CONS][NewCohort][fat]    = (QJ)/(1+QJ)*CONS_IW;

	  for (i=0; i<I_CONST_DIM; i++) popIDcard[CONS][NewCohort][i] = 0.0;
	}
      else NewCohort =  -1;
    }
#endif

  return;
}




/*===========================================================================*/

void	SetBpoints(double *env, population *pop, population *bpoints)

{

  return;
}




/*
 *===========================================================================
 *
 *			SPECIFICATION OF DERIVATIVES
 *
 *===========================================================================
 */

void	Gradient(double *env,	  population *pop,     population *ofs,
		 double *envgrad, population *popgrad, population *ofsgrad,
		 population *bpoints)

{
  register int		i;
  double		kappa = 0.0,  net_energy = 0.0;
  double		ingest = 0.0, mort = 0.0;
  double		tot_grazing = 0.0;

  UpdateIDcards(env, pop, 0);

  for (i=0; i<cohort_no[CONS]; i++)
    {
      popgrad[CONS][i][number] =  0.0;
      popgrad[CONS][i][age]    =  1.0;
      popgrad[CONS][i][bone]   =  0.0;
      popgrad[CONS][i][fat]    = -1.0;
      if (pop[CONS][i][fat] < TINY*pop[CONS][i][bone]) continue;

      mort	 = popIDcard[CONS][i][IDmort];
      kappa	 = popIDcard[CONS][i][IDkappa];
      net_energy = popIDcard[CONS][i][IDnet_energy];
      ingest	 = popIDcard[CONS][i][IDingest];

      popgrad[CONS][i][number] = -mort*pop[CONS][i][number];
      popgrad[CONS][i][bone]   = kappa*net_energy;
      popgrad[CONS][i][fat]    = (1-kappa)*net_energy;

      tot_grazing += ingest*pop[CONS][i][number];
    }

#if (PULSED_REPRO == 0)
  if (NewCohort >= 0)
    {
      double			tmp, nx, Hatching;

      nx = 1.5*(fmod(time, YEAR) - HATCH_START)/(HATCH_MAX - HATCH_START);

      if ((nx <= 0.0) || (nx >= 3.0)) 	Hatching = 0.0;
      else if (nx <= 1.0)		Hatching =                 0.5*nx*nx;
      else if (nx <= 2.0)		Hatching = -1.5 + 3.0*nx -     nx*nx;
      else				Hatching =  4.5 - 3.0*nx + 0.5*nx*nx;

      Hatching *= HATCH_SURV*TotalEggs/(2.0*(HATCH_MAX - HATCH_START)/3.0);

      // For numerical stability of the integration we use a formulation for
      // the dynamics of the boundary cohort that is different than the
      // standard one.
      // For the derivation of the derivatives describing the dynamics of the
      // i-state variables see eq. 3.11 of the reader on using the EBT approach
      // available at:
      //
      // http://www.science.uva.nl/~aroos/downloads/EBT/EBTsyllabus.pdf
      //
      // The derivatives for the dynamics of these i-state variables in case
      // the cohort is still empty result from a limiting argument for all
      // cohort variables to 0.
      popgrad[CONS][NewCohort][number] += Hatching;
      if (pop[CONS][NewCohort][number] < SMALL)
	{
	  popgrad[CONS][NewCohort][age]  *= 0.5;
	  popgrad[CONS][NewCohort][bone] *= 0.5;
	  popgrad[CONS][NewCohort][fat]  *= 0.5;
	}
      else
	{
	  tmp  = - pop[CONS][NewCohort][age];
	  tmp /= pop[CONS][NewCohort][number];
	  popgrad[CONS][NewCohort][age] += Hatching*tmp;

	  tmp  = (1/(1+QJ)*CONS_IW - pop[CONS][NewCohort][bone]);
	  tmp /= pop[CONS][NewCohort][number];
	  popgrad[CONS][NewCohort][bone] += Hatching*tmp;

	  tmp  = ((QJ)/(1+QJ)*CONS_IW - pop[CONS][NewCohort][fat]);
	  tmp /= pop[CONS][NewCohort][number];
	  popgrad[CONS][NewCohort][fat] += Hatching*tmp;
	}
    }
#endif

  envgrad[0] = 1.0;
  envgrad[1] = R*Temp_factor(time, RES_MIN_RATE)*(K-food) - tot_grazing/TOTVOL;
  envgrad[2] = 0.0;

  return;
}




/*
 *===========================================================================
 *
 *		SPECIFICATION OF BETWEEN COHORT CYCLE DYNAMICS
 *
 *===========================================================================
 */

void	InstantDynamics(double *env, population *pop, population *ofs)

{
  register int		i, nyoy;
  double		TotalRepro = 0.0, fec;


  if (food < MINFOOD) food = MINFOOD;		// Check negative food

  for (i=0, nyoy=0, YOYsurvival = 0.0; i<cohort_no[CONS]; i++)
    {
      // remove cohorts with no fat
      if (pop[CONS][i][fat] < TINY*pop[CONS][i][bone])
	pop[CONS][i][number] = 0.0;

      // remove very old cohorts
      if (pop[CONS][i][age] > (CONS_MAX_YEARS*YEAR))
	pop[CONS][i][number] = 0.0;

      // Store YOY survival for output
      if ((pop[CONS][i][age] < YEAR + SMALL) && (popIDcard[CONS][i][IDnbegin] > TINY))
	{
	  YOYsurvival = (nyoy*YOYsurvival + pop[CONS][i][number]/popIDcard[CONS][i][IDnbegin])/(nyoy + 1);
	  nyoy++;
	}

      // Store size at age 1 for output
      if (fabs(pop[CONS][i][age] - YEAR) < SMALL) SizeOYO = pop[CONS][i][bone];
    }

#if (PULSED_REPRO == 0)
  if (NewCohort >= 0)
    popIDcard[CONS][NewCohort][IDnbegin] = pop[CONS][NewCohort][number];
#endif

  TimeInYear = fmod(time, YEAR);

  // Fix gonads into eggs at the end of summer
  if (isequal(TimeInYear, SEASON) ||
      (isequal(SEASON, YEAR) && ((TimeInYear < SMALL) || (TimeInYear > (YEAR-SMALL)))))
    UpdateIDcards(env, pop, 1);

  // Eggs are spawned by adults at day SPAWN_DAY of the year
  if (isequal (TimeInYear, SPAWN_DAY) || 
      (isequal(SPAWN_DAY, 0.0) && ((TimeInYear < SMALL) || (TimeInYear > (YEAR-SMALL)))))
    {
      ReproCohorts = 0;
      TotalRepro   = 0.0;
      TotalEggs	   = 0.0;

      for (i=0; i<cohort_no[CONS]; i++)
	{
	  if ((pop[CONS][i][fat] < TINY*pop[CONS][i][bone]) || 
	      (pop[CONS][i][number] <= 0.0)) continue;

	  if (pop[CONS][i][bone] > XMAT)
	    {
	      // Compute resources invested in eggs at winter start
	      fec = popIDcard[CONS][i][IDfecundity]*(2.0*CONS_IW);

	      if (popIDcard[CONS][i][IDfecundity] < TINY) continue; 

	      if (pop[CONS][i][fat] > fec + TINY)
		{
		  TotalEggs += (popIDcard[CONS][i][IDfecundity]*pop[CONS][i][number]);
		  pop[CONS][i][fat] -=fec;

		  ReproCohorts++;
		  TotalRepro += pop[CONS][i][number];
		}
	      else
		{
		  // These individuals have starved away part all of their
		  // gonads during winter. They hence reproduce and die
		  TotalEggs += (pop[CONS][i][fat]*pop[CONS][i][number]/(2.0*CONS_IW));

		  ReproCohorts++;
		  TotalRepro += pop[CONS][i][number];
		  pop[CONS][i][number] = 0.0;
		}
	    }
	}

#if (PULSED_REPRO == 1)
      if (TotalEggs)
	{
	  NewCohort = AddCohorts(pop, CONS, 1);
	  pop[CONS][NewCohort][number] = HATCH_SURV*TotalEggs;
	  pop[CONS][NewCohort][age]    = SPAWN_DAY;
	  pop[CONS][NewCohort][bone]   = 1/(1+QJ)*CONS_IW;
	  pop[CONS][NewCohort][fat]    = (QJ)/(1+QJ)*CONS_IW;

	  popIDcard[CONS][NewCohort][IDnbegin]    = HATCH_SURV*TotalEggs;
	  popIDcard[CONS][NewCohort][IDfecundity] = 0.0;

	  SetStepSize(SMALL);
	}
      NewCohort = -1;
#endif

      if (TotalRepro) MeanFecundity = TotalEggs/TotalRepro;
      else MeanFecundity = 0.0;
    }

  UpdateIDcards(env, pop, 0);

  return;
}




/*
 *===========================================================================
 *
 *			SPECIFICATION OF OUTPUT VARIABLES
 *
 *===========================================================================
 */

void	DefineOutput(double *env, population *pop, double *output)

{
  register int		i;
  double		num, wght, biom;

  /*
   * OUTPUT VARIABLES:
   * [ 0]	 2: Food density
   * [ 1]	 3: YOY
   * [ 2]	 4: YOY biomass
   * [ 3]	 5: 1+
   * [ 4]	 6: 1+ biomass
   * [ 5]	 7: Juvenile 1+
   * [ 6]	 8: Juvenile 1+ biomass
   * [ 7]	 9: Adult 1+
   * [ 8]	10: Adult 1+  biomass
   * [ 9]	11: YOY survival
   * [10]	12: OYO body size
   * [11]	13: # of reproducing cohorts
   * [12]	14: Mean fecundity
   * [13]	15: Temp factor
   */

  UpdateIDcards(env, pop, 0);

#if (BIFURCATION == 1)
  LabelState(CONS, "\\8m\\4 = %.4f   T = %.0f", parameter[BifParIndex], env[0]);
#endif /* BIFURCATION */

  output[0] = food;
  for (i=0; i<cohort_no[CONS]; i++)
    {
      num  = pop[CONS][i][number];
      wght = pop[CONS][i][bone]+pop[CONS][i][fat];
      biom = wght*pop[CONS][i][number];
      
      if (pop[CONS][i][age] < YEAR - SMALL)
	{
	  output[ 1] += num;
	  output[ 2] += biom;
	}
      else
	{
	  output[ 3] += num;
	  output[ 4] += biom;

	  if (pop[CONS][i][bone] < XMAT)
	    {
	      output[ 5] += num;
	      output[ 6] += biom;
	    }
	  else
	    {
	      output[ 7] += num;
	      output[ 8] += biom;
	    }
	}
    }

  output[ 9] = YOYsurvival;
  output[10] = SizeOYO;
  output[11] = ReproCohorts;
  output[12] = MeanFecundity;
  output[13] = Temp_factor(time, CONS_MIN_RATE);

  output[14] = TotalEggs;

  return;
}



/*==============================================================================*/

static void	UpdateIDcards(double *env, population *pop, int setfec)

{
  register int		i;
  double		mass = 0.0, weight = 0.0;
  double		arate = 0.0, htime = 0.0, ingest = 0.0, maint = 0.0;
  double		q = 0.0, kappa = 0.0, net_energy = 0.0;
  double		mort = 0.0, fecundity  = 0.0;
  double		tfac;

  tfac = Temp_factor(time, CONS_MIN_RATE);

  for (i=0; i<cohort_no[CONS]; i++)
    {
      mass   = pop[CONS][i][bone]*(1 + QJ);
      weight = pop[CONS][i][bone] + pop[CONS][i][fat];

      arate  = AR_CON*pow(mass/AR_WOPT*exp(1-mass/AR_WOPT), AR_EXP);
      htime  = DIGT_CON*pow(mass, DIGT_EXP);
      maint  = tfac*MR_CON*pow(weight, MR_EXP);

      ingest = tfac*arate*food/(1+arate*htime*food);
      net_energy = (CONVEFF*ingest - maint);

      if (pop[CONS][i][bone] < XMAT) q = QJ;
      else q = QA;
      if (net_energy > 0.0)
	{
	  if (q*pop[CONS][i][bone] - pop[CONS][i][fat] < 0.0)
	    kappa = 1/(q+1);
	  else
	    kappa = pop[CONS][i][fat]/((1+q)*q*pop[CONS][i][bone]);
	}
      else
	kappa = 0.0;

      if (X_MU > TINY)
	mort = tfac*(MU_BACK + MU_J*exp(-pop[CONS][i][bone]/X_MU));
      else
	{
	  if (pop[CONS][i][bone] < XMAT)
	    mort  = tfac*(MU_J+MU_BACK);
	  else
	    mort  = tfac*MU_BACK;
	}

      if (pop[CONS][i][fat] < QS*pop[CONS][i][bone])
	mort += tfac*MU_S*(QS*pop[CONS][i][bone]/pop[CONS][i][fat] - 1.0);

      if ((pop[CONS][i][bone] > XMAT) &&
	  (pop[CONS][i][fat]  > QJ*pop[CONS][i][bone]))
	fecundity = (pop[CONS][i][fat]-QJ*pop[CONS][i][bone])/(2.0*CONS_IW);
      else
	fecundity = 0.0;

      popIDcard[CONS][i][IDmass]	= mass;
      popIDcard[CONS][i][IDweight]	= weight;
      popIDcard[CONS][i][IDarate]	= arate;
      popIDcard[CONS][i][IDhtime]	= htime;
      popIDcard[CONS][i][IDmaint]	= maint;
      popIDcard[CONS][i][IDingest]	= ingest;
      popIDcard[CONS][i][IDnet_energy]	= net_energy;
      popIDcard[CONS][i][IDkappa]	= kappa;
      popIDcard[CONS][i][IDmort]	= mort;
      popIDcard[CONS][i][IDfatratio]	= pop[CONS][i][fat]/pop[CONS][i][bone];

      if (setfec)
	popIDcard[CONS][i][IDfecundity]	= fecundity;
    }

  return;
}



/*==============================================================================*/

static double	Temp_factor(double t, double min_fac)

{
  double		tau, temp_fac;

  tau = fmod(t+TINY, YEAR);			

#if (STEP_TEMP == 1)

  if (tau <= SEASON)
    temp_fac = 1;						// in summer
  else
    temp_fac = min_fac;						// in winter

#else
  double		tmp, ct;
  
  if (tau <= SEASON)
    {
      tmp = sin(2*M_PI*tau/(2*SEASON));				// in summer
      ct  = MIN_TEMP + 0.5*(MAX_TEMP - MIN_TEMP)*(1.0 + pow( tmp, THETA));  
     
    }	
  else
    {
      tmp = sin (2*M_PI*(tau-SEASON) /(2*(YEAR-SEASON)));	// in winter
      ct  = MIN_TEMP + 0.5*(MAX_TEMP - MIN_TEMP)*(1.0 - pow(tmp, THETA));
    }
		
  tmp  = 1.0/MAX_TEMP - 1.0/ct;
  tmp /= 1.0/MAX_TEMP - 1.0/MIN_TEMP;				// Arrhenius relation

  temp_fac = pow(min_fac, tmp);
#endif

  return temp_fac;

}

/*==============================================================================*/
